@@ -374,17 +374,30 @@ read_desktop_entries_in_dir (GHashTable *clients, const gchar *path)
       Client *hash_client;
       SmProp *prop;
       gchar *desktop_file;
+      gchar *hash_key;
       GnomeDesktopItem *ditem;
 
       if (!g_str_has_suffix (name, ".desktop"))
         continue;
 
+      /* an entry with a filename cancels any previous entry with the same
+       * filename */
+      if (g_hash_table_lookup_extended (clients, name,
+                                        (gpointer *) &hash_key,
+                                        (gpointer *) &hash_client))
+      {
+        g_hash_table_remove (clients, name);
+        g_free (hash_key);
+	free_client (hash_client);
+      }
+
+
       desktop_file = g_build_filename (path, name, NULL);
       ditem = gnome_desktop_item_new_from_file (desktop_file, GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS, NULL);
       if (ditem != NULL)
         {
 	  const gchar *exec_string;
-	  gchar **argv, *hash_key;
+	  gchar **argv;
 	  gint argc, i;
 
 	  exec_string = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC);
@@ -497,13 +510,6 @@ read_desktop_entries_in_dir (GHashTable *clients, const gchar *path)
 
 	  APPEND (client->properties, prop);
 
-	  if (g_hash_table_lookup_extended (clients, name, (gpointer *) &hash_key, (gpointer *) &hash_client))
-	    {
-	      g_hash_table_remove (clients, name);
-	      g_free (hash_key);
-	      free_client (hash_client);
-	    }
-
 	  /* check the X-GNOME-Autostart-enabled field */
 	  if (gnome_desktop_item_attr_exists (ditem, "X-GNOME-Autostart-enabled"))
 	    {
@@ -543,25 +549,30 @@ read_autostart_dirs (void)
   const char * const * system_dirs;
   char *path;
   gint i;
+  gint len;
   GHashTable *clients;
   GSList *list = NULL;
 
   clients = g_hash_table_new (g_str_hash, g_str_equal);
 
-  /* read directories */
-  system_dirs = g_get_system_data_dirs ();
-  for (i = 0; system_dirs[i] != NULL; i++)
+  /* support old place (/etc/xdg/autostart) */
+  system_dirs = g_get_system_config_dirs ();
+  for (len = 0; system_dirs[len] != NULL; len++);
+
+  for (i = len - 1; i >= 0; i--)
     {
-      path = g_build_filename (system_dirs[i], "gnome", "autostart", NULL);
+      path = g_build_filename (system_dirs[i], "autostart", NULL);
       read_desktop_entries_in_dir (clients, path);
       g_free (path);
     }
 
-  /* support old place (/etc/xdg/autostart) */
-  system_dirs = g_get_system_config_dirs ();
-  for (i = 0; system_dirs[i] != NULL; i++)
+  /* read directories */
+  system_dirs = g_get_system_data_dirs ();
+  for (len = 0; system_dirs[len] != NULL; len++);
+
+  for (i = len - 1; i >= 0; i--)
     {
-      path = g_build_filename (system_dirs[i], "autostart", NULL);
+      path = g_build_filename (system_dirs[i], "gnome", "autostart", NULL);
       read_desktop_entries_in_dir (clients, path);
       g_free (path);
     }
@@ -743,8 +754,10 @@ read_session (const char *name)
 	GString *args = g_string_new ((const char *) prop->vals[0].value);
 
 	for (i = 1; i < prop->num_vals; i++) {
+	  gchar *quoted = g_shell_quote (prop->vals[i].value);
 	  args = g_string_append_c (args, ' ');
-	  args = g_string_append (args, (const char *) prop->vals[i].value);
+	  args = g_string_append (args, quoted);
+	  g_free (quoted);
 	}
 
 	ditem = gnome_desktop_item_new ();
